@@ -49,6 +49,10 @@ pub(crate) fn tokenizer<P: AsRef<Path> + Clone>(
     }
 }
 
+lazy_static! {
+    static ref RULES: Mutex<HashMap<Option<PathBuf>, Arc<Rules>>> = Mutex::new(HashMap::new());
+}
+
 fn rules_inner<P: AsRef<Path>>(override_path: Option<P>) -> Result<Rules> {
     info!("🧮 Loading rules...");
     let rules = if let Some(override_path) = override_path.as_ref() {
@@ -61,10 +65,20 @@ fn rules_inner<P: AsRef<Path>>(override_path: Option<P>) -> Result<Rules> {
     Ok(rules)
 }
 
-pub(crate) fn rules<P: AsRef<Path> + Clone>(override_path: Option<P>) -> Result<Rules> {
-    // XXX TODO right now `Rules` is not copy and only used in one place
-    // so this is fine for now
-    rules_inner(override_path)
+pub(crate) fn rules<P: AsRef<Path> + Clone>(override_path: Option<P>) -> Result<Arc<Rules>> {
+    match RULES
+        .lock()
+        .unwrap()
+        .entry(override_path.clone().map(|x| x.as_ref().to_path_buf()))
+    {
+        Entry::Occupied(occupied) => Ok(occupied.get().clone()),
+        Entry::Vacant(empty) => {
+            let rules = rules_inner(override_path)?;
+            let rules = Arc::new(rules);
+            empty.insert(rules.clone());
+            Ok(rules)
+        }
+    }
 }
 
 use crate::Range;
@@ -102,13 +116,20 @@ where
             // Also allows i.e. `ink!'s` to be detected as a single
             // token.
             while let Some(token) = iter.next() {
-                let char_range = token.span().char().clone();
+                let char_range = dbg!(&token).span().char().clone();
 
                 let space = iter
                     .peek()
                     .map(|upcoming| upcoming.has_space_before())
                     .unwrap_or(false);
                 let s = token.word().as_str();
+                // TODO workaround for a bug in srx
+                // TODO that does not split `[7f` after `[` as expected
+                if s.starts_with('[') && char_range.len() > 1 {
+                    acc.push((char_range.start)..(char_range.start + 1));
+                    acc.push((char_range.start + 1)..(char_range.end));
+                    continue;
+                }
                 let belongs_to_genitive_s = match s {
                     "(" | ")" | r#"""# => false,
                     _ => true,
@@ -153,6 +174,8 @@ where
 
 #[cfg(test)]
 mod tests {
+    use crate::sub_chars;
+
     use super::*;
 
     #[test]
@@ -277,4 +300,18 @@ mod tests {
                 assert_eq!(is, expect);
             });
     }
+
+    #[test]
+    fn tokenize_square_bracket_foo_square_bracket() {
+        let text = r#"[1337]"#;
+        let tok = tokenizer::<PathBuf>(None).unwrap();
+        let ranges = apply_tokenizer(&tok, text);
+
+        ranges
+            .zip([0_usize..1, 1..5, 5..6].iter().cloned())
+            .for_each(|(is, expect)| {
+                dbg!((sub_chars(text, is.clone()), sub_chars(text, expect.clone())));
+                assert_eq!(is, expect);
+            });
+    }
 }
